﻿//------------------------------------------------------------------------------
// This is a simple application to demonstrate the usage of the CarTFT.com FM
// radio module Radiator plug-in outside radiator.
// For a full documentation about Radiator plug-ins, please take a look at the Radiator
// SDK which can be downloaded at http://flesko.cz/programs.html
//
// (C) Copyright 2009-2010 bay CarTFT.com / Zoran Horvat.
//
// Note: To run this application you have to have a "Plugins" folder within the output 
// folder containing the "plugins.ini" file, which holds the settings (like COM port)
// needed to initialize the plug-in.
//
// In contrast to a standard Radiator plug-in, this plug-in has 3 additional methods
// exported:
//      GetRdsStation   - reads the station name using RDS
//      GetRdsText      - reads the radio text using RDS
//      GetRdsTimer     - reads the current time using RDS 
//------------------------------------------------------------------------------
using System;
using System.Globalization;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;

namespace RadiatorPluginTest
{
    public partial class Form1 : Form
    {
        private int frequency = 10640;

        #region PInvoke
        //------------------------------------------------------------------------------
        // This section defines the PInvoke functions as they can be found in 
        // CarTFTFM.dll
        //------------------------------------------------------------------------------
        //------------------------------------------------------------------------------
        // structure definition for the RDS time
        //------------------------------------------------------------------------------
        public struct RdsTime
        {
            public short Year;
            public short Month;
            public short Day;
            public short DayOfWeek;

            public short Hour;
            public short Minute;
            public short Second;
        }
        
        [Flags]
        public enum ModuleCapabilities
        {
            FM_TUNE	                = 1,               // Supports setting frequency. 
            FM_TUNEMUTED			= 2,               // Supports setting frequency "silently". Output is muted, but frequency is set. Then just unmute to hear it. 
            FM_MUTEUNMUTE			= 4,               // Supports muting/unmuting. 
            FM_GETVOLUME			= 8,               // Supports GetVolume function. 
            FM_SETVOLUMEBYVALUE		= 16,              // Supports setting volume "digitally" by setting a value. 
            FM_SETVOLUMEUPDOWN		= 32,              // Supports setting volume up and down by discrete steps (this is how Radiator works now). 
            FM_BASSTREBLE			= 64,              // Supports bass & treble. 
            FM_ISSTEREO				= 128,             // Supports stereo identification. 
            FM_SETSTEREO			= 256,             // Also called forced stereo or mono. 
            FM_GETSIGNAL			= 512,             // Station is/is not tuned to signal. 
            FM_AMRANGE				= 1024,            // Supports AM range (not prepared in Radiator now). 
            FM_RDS					= 2048,            // Supports RDS - not prepared in any way yet ;-). 
            FM_SCANSTATION			= 4096
        }

        //------------------------------------------------------------
        // Mandantory functions
        //------------------------------------------------------------
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        [return: MarshalAs(UnmanagedType.LPStr)]
        public static extern string GetModuleName();
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern int GetModuleInfo();
        
        /// <summary>
        /// Init function. Must be called on startup. 
        /// </summary>
        /// <returns>Returns true if the module was initialized successfully, false otherwise.</returns>
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern bool HWInit();
        /// <summary>
        /// Deinit function. Must be called when the application gets shut down.
        /// </summary>
        /// <returns>Returns True if successful, false otherwise.</returns>
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern bool HWDeInit();
        //------------------------------------------------------------
        // Optional functions
        //------------------------------------------------------------
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        [return: MarshalAs(UnmanagedType.LPStr)]
        public static extern string GetModuleComment();
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern void TuneFreq(int Freq);
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern void TuneFreqMuted(int Freq);
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern void SetMute(byte Mute);
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern int ScanStation(bool DirectionUp, int FreqToSearchFrom);
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern short GetVolume();
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern void SetVolume(short Left, short Right);
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern void VolumeUpDown(int Step);
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern void SetBass(short Bass);
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern short GetBass();
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern void SetTreble(short Trebble);
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern short GetTreble();
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern bool IsStereo();
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern void SetStereo(byte Stereo);
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern short GetSignal();
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern bool GetRdsStation(out int PiCode, StringBuilder Station);
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern bool GetRdsText(StringBuilder Radiotext);
        
        [DllImport("CarTFTFM.dll", CharSet = CharSet.Auto)]
        public static extern bool GetRdsTime(out RdsTime Time);

        #endregion

        public Form1()
        {
            InitializeComponent();
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            //------------------------------------------------------------------------------
            // IMPORTANT: you have to initialize the plug-in. 
            // If this step fails please check the following steps:
            //   - is the CarTFT.com FM receiver connected to the USB port
            //   - do you have a folder named "Plugins" containing the "plugins.ini" file
            //   - is the COM port setting in plugins.ini (section [CarTFTFM]) correct
            //------------------------------------------------------------------------------
            if(HWInit())
                ShowMessage("Init successful.");
            else
            {
                ShowMessage("Init has failed!");
                return;
            }
            short volume = GetVolume();
            trackBarVolume.Value = volume;
            textBoxFreq.Text = ((double)frequency/100).ToString("0.00", CultureInfo.InvariantCulture);
        }

        private void ShowMessage(string s)
        {
            textBox1.AppendText(s + "\r\n");
        }

        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            if (!HWDeInit())
                MessageBox.Show(this, "The deinit has failed!");
        }

        private void buttonModulName_Click(object sender, EventArgs e)
        {
            //------------------------------------------------------------------------------
            // check the module name
            //------------------------------------------------------------------------------
            ShowMessage(string.Format("Modul name: {0}", GetModuleName()));
        }

        private void trackBarVolume_ValueChanged(object sender, EventArgs e)
        {
            //------------------------------------------------------------------------------
            // set the volume
            //------------------------------------------------------------------------------
            SetVolume((short)trackBarVolume.Value, (short)trackBarVolume.Value);
        }

        private void buttonTuneFreq_Click(object sender, EventArgs e)
        {
            //------------------------------------------------------------------------------
            // tune the frequency set in the text box (i.e. "96.1")
            //------------------------------------------------------------------------------
            if(string.IsNullOrEmpty(textBoxFreq.Text))
                return;
            frequency = (int)(double.Parse(textBoxFreq.Text, CultureInfo.InvariantCulture)*1000);
            TuneFreq(frequency);
        }

        private void buttonTuneDown_Click(object sender, EventArgs e)
        {
            //------------------------------------------------------------------------------
            // scan for the previous radio station
            //------------------------------------------------------------------------------
            frequency = ScanStation(false, 0);
            textBoxFreq.Text = ((double)frequency / 1000).ToString("0.00",CultureInfo.InvariantCulture);
        }

        private void buttonTuneUp_Click(object sender, EventArgs e)
        {
            //------------------------------------------------------------------------------
            // scan for the next radio station
            //------------------------------------------------------------------------------
            frequency = ScanStation(true, 0);
            textBoxFreq.Text = ((double)frequency/1000).ToString("0.00",CultureInfo.InvariantCulture);
        }

        private void buttonRdsStation_Click(object sender, EventArgs e)
        {
            //------------------------------------------------------------------------------
            // get the radio station name (RDS)
            //------------------------------------------------------------------------------
            StringBuilder station = new StringBuilder(9);
            int piCode;
            GetRdsStation(out piCode, station);
            ShowMessage(string.Format("RDS station: {0}", station));
        }

        private void buttonRdsText_Click(object sender, EventArgs e)
        {
            //------------------------------------------------------------------------------
            // get the radio text (RDS)
            //------------------------------------------------------------------------------
            StringBuilder text = new StringBuilder(250);
            GetRdsText(text);
            ShowMessage(string.Format("RDS text: {0}", text));
        }

        private void buttonTime_Click(object sender, EventArgs e)
        {
            //------------------------------------------------------------------------------
            // get the current time (RDS)
            //------------------------------------------------------------------------------
            RdsTime rdsTime = new RdsTime();
            GetRdsTime(out rdsTime);
            ShowMessage(string.Format("RDS time: {0}:{1}:{2}  {3}-{4}-{5}", rdsTime.Hour, rdsTime.Minute, rdsTime.Second,
                                      rdsTime.Year, rdsTime.Month, rdsTime.DayOfWeek));
        }

        private void buttonModuleInfo_Click(object sender, EventArgs e)
        {
            //------------------------------------------------------------------------------
            // get the modul info (i.e. shows what ModuleCapabilities it has)
            //------------------------------------------------------------------------------
            int modInfo = GetModuleInfo();
            ShowMessage(string.Format("Module info: 0x{0}", modInfo.ToString("X")));
        }
    }
}
